///////////////////////////////////////////////////////////////////////////////
// File: FragmentEchoingSocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2011 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "FragmentEchoingSocketServer.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/Exception.h"
#include "JetByteTools/CoreTools/ToBool.h"

#include "JetByteTools/IOTools/Buffer.h"
#include "JetByteTools/IOTools/BufferChain.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#include "JetByteTools/WebSocketTools/AutoDetectProtocolHandler.h"
#include "JetByteTools/WebSocketTools/Headers.h"
#include "JetByteTools/WebSocketTools/HyBi/ProtocolHandler.h"
#include "JetByteTools/WebSocketTools/HyBi/CloseStatusCodes.h"
#include "JetByteTools/WebSocketTools/HyBi/RequestHeaders.h"

#pragma hdrstop

#include "JetByteTools/CoreTools/StringConverter.h"

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::ToString;
using JetByteTools::Core::_tstring;
using JetByteTools::Core::OutputEx;
using JetByteTools::Core::DumpData;
using JetByteTools::Core::CException;
using JetByteTools::Core::CStringConverter;
using JetByteTools::Core::IIndexedOpaqueUserData;
using JetByteTools::Core::ToBool;

using JetByteTools::IO::IAllocateBuffers;
using JetByteTools::IO::IBuffer;
using JetByteTools::IO::IIOPool;
using JetByteTools::IO::CBufferChain;
using JetByteTools::IO::CSmartBuffer;

using JetByteTools::Socket::CStreamSocketServerEx;
using JetByteTools::Socket::IStreamSocket;
using JetByteTools::Socket::IAllocateSequencedStreamSockets;
using JetByteTools::Socket::IFullAddress;
using JetByteTools::Socket::IAddress;
using JetByteTools::Socket::ListenBacklog;
using JetByteTools::Socket::NoZeroByteRead;
using JetByteTools::Socket::ILimitConnections;
using JetByteTools::Socket::ConnectionClosureReason;

using JetByteTools::WebSocket::CHeaders;
using JetByteTools::WebSocket::MessageStatus;
using JetByteTools::WebSocket::MessageType;
using JetByteTools::WebSocket::MessageTypeText;
using JetByteTools::WebSocket::MessageTypeBinary;
using JetByteTools::WebSocket::ConnectionEstablishmentResult;
using JetByteTools::WebSocket::ConnectionEstablished;
using JetByteTools::WebSocket::IProtocolHandler;
using JetByteTools::WebSocket::MessageStatus;
using JetByteTools::WebSocket::MessageStatusComplete;

using JetByteTools::WebSocket::HyBi::CProtocolHandler;
using JetByteTools::WebSocket::HyBi::IWebSocket;
using JetByteTools::WebSocket::HyBi::CloseStatusNormal;
using JetByteTools::WebSocket::HyBi::CRequestHeaders;

using std::string;

///////////////////////////////////////////////////////////////////////////////
// CFragmentEchoingSocketServer
///////////////////////////////////////////////////////////////////////////////

CFragmentEchoingSocketServer::CFragmentEchoingSocketServer(
   const bool webSocketMessageAccumulation,
   const IFullAddress &address,
   const ListenBacklog listenBacklog,
   IIOPool &pool,
   IAllocateSequencedStreamSockets &socketAllocator,
   IAllocateBuffers &bufferAllocator,
   ILimitConnections &connectionLimiter)
   :  CStreamSocketServerEx(address, listenBacklog, *this, pool, socketAllocator, bufferAllocator, NoZeroByteRead, connectionLimiter),
      m_protocolHandlerIndex(socketAllocator.RequestUserDataSlot(_T("m_protocolHandlerIndex"))),
      m_userDataIndex(socketAllocator.RequestUserDataSlot(_T("m_userDataIndex"))),
      m_webSocketMessageAccumulation(webSocketMessageAccumulation),
      m_bufferAllocator(bufferAllocator),
      m_protocolHandlerAllocator(bufferAllocator)
{

}

CFragmentEchoingSocketServer::~CFragmentEchoingSocketServer()
{
   WaitForShutdownToComplete();
}

void CFragmentEchoingSocketServer::OnConnectionEstablished(
   IStreamSocket &socket,
   const IAddress & /*address*/)
{
   Output(_T("OnConnectionEstablished"));

   socket.SetUserData(m_userDataIndex, 0);

   const DWORD flags = (m_webSocketMessageAccumulation ? CProtocolHandler::AccumulateCompleteMessage : 0);

   CProtocolHandler *pHandler = m_protocolHandlerAllocator.Allocate(*this, socket, flags);

   socket.SetUserPointer(m_protocolHandlerIndex, pHandler);

   pHandler->AcceptHandshake();
}

void CFragmentEchoingSocketServer::OnReadCompleted(
   IStreamSocket &socket,
   CSmartBuffer &buffer)
{
   try
   {
      //DEBUG_ONLY(Output(_T("WebSocket Data: \r\n") + DumpData(buffer.GetMemory(), buffer.GetUsed(), 60, true)));

      IProtocolHandler *pHandler = reinterpret_cast<IProtocolHandler *>(socket.GetUserPointer(m_protocolHandlerIndex));

      pHandler->OnDataReceived(buffer);
   }
   catch(const CException &e)
   {
      OutputEx(_T("ReadCompleted - Exception - ") + e.GetDetails());
      socket.AbortConnection();
   }
   catch(...)
   {
      OutputEx(_T("ReadCompleted - Unexpected exception"));
      socket.AbortConnection();
   }
}

void CFragmentEchoingSocketServer::OnConnectionReset(
   IStreamSocket &socket,
   const DWORD /*lastError*/)
{
   IProtocolHandler *pHandler = static_cast<IProtocolHandler *>(socket.GetUserPointer(m_protocolHandlerIndex));

   pHandler->OnConnectionClosed();
}

void CFragmentEchoingSocketServer::OnConnectionClosure(
   IStreamSocket &socket,
   const ConnectionClosureReason /*reason*/)
{
   IProtocolHandler *pHandler = reinterpret_cast<IProtocolHandler *>(socket.GetUserPointer(m_protocolHandlerIndex));

   pHandler->OnConnectionClosed();
}

void CFragmentEchoingSocketServer::OnSocketReleased(
   IIndexedOpaqueUserData &userData)
{
   Output(_T("OnSocketReleased"));

   IProtocolHandler *pHandler = reinterpret_cast<IProtocolHandler *>(userData.GetUserPointer(m_protocolHandlerIndex));

   delete pHandler;
}

ConnectionEstablishmentResult CFragmentEchoingSocketServer::OnConnectionHandshake(
   IIndexedOpaqueUserData & /*userData*/,
   const string &uri,
   const bool /*secure*/,
   const CHeaders &requestHeaders,
   CHeaders &responseHeaders)
{
   Output(_T("OnConnectionHandshake - : ") + CStringConverter::AtoT(uri));

   CRequestHeaders::DumpRequestHeadersToDebug(requestHeaders);

   const string versionHeader = "sec-websocket-version";

   if (requestHeaders.Contains(versionHeader))
   {
      const string version = requestHeaders.GetValue(versionHeader);

      Output(_T("Protocol Version: HyBi: ") + CStringConverter::AtoT(version));
   }
   else
   {
      throw CException(_T("CHyBiSocketServer::OnConnectionHandshake()"), _T("Protocol version unknown"));
   }

   CRequestHeaders::AddOriginHeaders(responseHeaders, requestHeaders);

   return ::ConnectionEstablished;
}

void CFragmentEchoingSocketServer::OnConnectionEstablished(
   IWebSocket &socket,
   const string &uri,
   const bool /*secure*/)
{
   Output(_T("OnConnectionEstablished: ") + CStringConverter::AtoT(uri));

   socket.TryRead();
}

void CFragmentEchoingSocketServer::OnOutboundConnectionEstablished(
   IWebSocket & /*socket*/,
   const CHeaders & /*responseHeaders*/)
{
   throw CException(
      _T("CFragmentEchoingSocketServer::OnOutboundConnectionEstablished()"),
      _T("Unexpected: we don't create outbound connections"));
}

void CFragmentEchoingSocketServer::OnData(
   IWebSocket & /*socket*/,
   const _tstring & /*text*/,
   const MessageStatus /*status*/,
   const __int64 /*messageBytesOutstanding*/)
{
   // For this to be called you need to specify CProtocolHandler::DispatchTextAsStrings in the flags that are passed
   // during construction

   throw CException(
      _T("CFragmentEchoingSocketServer::OnData()"),
      _T("Unexpected: we don't handle text frames"));
}

void CFragmentEchoingSocketServer::OnData(
   IWebSocket &socket,
   CSmartBuffer &buffer,
   const MessageType type,
   const MessageStatus status,
   const __int64 messageBytesOutstanding)
{
   //DEBUG_ONLY(Output(_T("OnData - ") + ToString(buffer->GetUsed()) + _T(" bytes\r\n") + DumpData(buffer->GetMemory(), buffer->GetUsed())));

   const bool echoing = ToBool(socket.GetUserData(m_userDataIndex));

   if (echoing)
   {
      if (status == MessageStatusComplete && messageBytesOutstanding == 0)
      {
         socket.StartNewFragment(buffer->GetUsed(), buffer, true);

         socket.SetUserData(m_userDataIndex, false);
      }
      else
      {
         socket.StartNewFragment(buffer->GetUsed(), buffer);
      }
   }
   else 
   {
      socket.StartFragmentedMessage(type, buffer->GetUsed(), buffer);

      socket.SetUserData(m_userDataIndex, true);
   }

   socket.TryRead();
}

void CFragmentEchoingSocketServer::OnPingResponse(
   IWebSocket & /*socket*/,
   const BYTE * /*pData*/,
   const BYTE /*length*/)
{
   throw CException(
      _T("CFragmentEchoingSocketServer::OnPingResponse()"),
      _T("Unexpected: we don't send pings"));
}

void CFragmentEchoingSocketServer::OnClientClose(
   IWebSocket &socket,
   const WORD status,
   const _tstring &reason)
{
   Output(_T("OnClientClose:") + ToString(status) + _T(": ") + reason);

   socket.Close(CloseStatusNormal);
}

void CFragmentEchoingSocketServer::OnError(
   IWebSocket & /*socket*/,
   const _tstring &message)
{
   OutputEx(_T("OnError:") + message);
}

void CFragmentEchoingSocketServer::OnClosed(
   IWebSocket & /*socket*/,
   const WORD status)
{
   Output(_T("OnClosed:") + ToString(status));
}

///////////////////////////////////////////////////////////////////////////////
// End of file: FragmentEchoingSocketServer.cpp
///////////////////////////////////////////////////////////////////////////////
