#pragma once
#ifndef GATEWAY_SERVER_CONNECTION_INCLUDED__
#define GATEWAY_SERVER_CONNECTION_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: Connection.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2011 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CoreTools/AtomicBool.h"
#include "JetByteTools/CoreTools/LockableObject.h"
#include "JetByteTools/CoreTools/ReferenceCount.h"

#include "JetByteTools/SocketTools/FullAddress.h"
#include "JetByteTools/SocketTools/SmartStreamSocket.h"

#include "JetByteTools/WebSocketTools/HyBi/SmartWebSocket.h"

#include <string>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Socket
   {
      class ICreateStreamSocketConnections;
      class IStreamSocket;
      class IAddress;
   }

   namespace IO
   {
      class IBuffer;
   }

   namespace WebSocket
   {
      namespace HyBi
      {
         class IWebSocket;
      }
   }
}

///////////////////////////////////////////////////////////////////////////////
// CConnection
///////////////////////////////////////////////////////////////////////////////

class CConnection
{
   public :

      CConnection(
         JetByteTools::Socket::ICreateStreamSocketConnections &connectionCreator,
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         const JetByteTools::Socket::IAddress &remoteAddress);

      CConnection(
         const CConnection &rhs) = delete;

      CConnection &operator=(
         const CConnection &rhs) = delete;

      void Release();

      void OnConnectionEstablished();

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address);

      void OnConnectionFailed(
         DWORD lastError);

      void OnReadCompleted(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      void OnConnectionClosed(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket);

      void OnConnectionClosed(
         JetByteTools::Socket::IStreamSocket &socket);

   private :

      JetByteTools::WebSocket::HyBi::CSmartWebSocket GetLocalSocket();

      JetByteTools::WebSocket::HyBi::CSmartWebSocket DetachLocalSocket();

      JetByteTools::Socket::CSmartStreamSocket GetRemoteSocket();

      JetByteTools::Socket::CSmartStreamSocket DetachRemoteSocket();

      ~CConnection();

      void AddRef();

      mutable JetByteTools::Core::CLockableObject m_lock;

      JetByteTools::Socket::ICreateStreamSocketConnections &m_connectionCreator;

      JetByteTools::WebSocket::HyBi::IWebSocket *m_pLocalSocket;

      JetByteTools::Socket::IStreamSocket *m_pRemoteSocket;

      const JetByteTools::Socket::CFullAddress m_remoteAddress;

      JetByteTools::Core::CReferenceCount m_ref;
};

#endif // GATEWAY_SERVER_CONNECTION_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: Connection.h
///////////////////////////////////////////////////////////////////////////////
