#pragma once
#ifndef GATEWAY_SERVER_SOCKET_SERVER_INCLUDED__
#define GATEWAY_SERVER_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2011 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "JetByteTools/CoreTools/IIndexedOpaqueUserData.h"

#include "JetByteTools/SocketTools/FullAddress.h"
#include "JetByteTools/SocketTools/IAllocateSequencedStreamSockets.h"

#include "JetByteTools/WebSocketTools/HyBi/IWebSocketServer.h"
#include "JetByteTools/WebSocketTools/HyBi/ProtocolHandlerAllocator.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::Socket::CStreamSocketServerEx,
   private CStreamSocketServerExCallback,
   private JetByteTools::WebSocket::HyBi::IWebSocketServer
{
   public :

      CSocketServer(
         const JetByteTools::Socket::IFullAddress &remoteAddress,
         const JetByteTools::Socket::IFullAddress &address,
         JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      CSocketServer(
         const CSocketServer &rhs) = delete;

      ~CSocketServer() override;

      CSocketServer &operator=(
         const CSocketServer &rhs) = delete;

   private :

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnPreOutgoingConnect(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const JetByteTools::Socket::IAddress &address,
         const void *pUserData) override;

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnOutgoingConnectionFailed(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         DWORD lastError) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionClientClose(
          JetByteTools::Socket::IStreamSocket &socket) override;

      void OnConnectionReset(
          JetByteTools::Socket::IStreamSocket &socket,
          DWORD lastError) override;

      void OnConnectionClosure(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::Socket::ConnectionClosureReason reason) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      /// Implement IAcceptWebSocketConnections

      JetByteTools::WebSocket::ConnectionEstablishmentResult OnConnectionHandshake(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const std::string &uri,
         bool secure,
         const JetByteTools::WebSocket::CHeaders &requestHeaders,
         JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      // Implement HyBi IWebSocketServer

      void OnOutboundConnectionEstablished(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      void OnConnectionEstablished(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const std::string &uri,
         bool secure) override;

      void OnData(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::Core::_tstring &text,
         JetByteTools::WebSocket::MessageStatus status,
         __int64 messageBytesOutstanding) override;

      void OnData(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         JetByteTools::WebSocket::MessageType type,
         JetByteTools::WebSocket::MessageStatus status,
         __int64 messageBytesOutstanding) override;

      void OnPingResponse(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const BYTE *pData,
         BYTE length) override;

      void OnClientClose(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         WORD status,
         const JetByteTools::Core::_tstring &text) override;

      void OnError(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::Core::_tstring &message) override;

      void OnClosed(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         WORD status) override;

      const JetByteTools::Socket::CFullAddress m_remoteAddress;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_protocolHandlerIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_userDataIndex;

      JetByteTools::IO::IAllocateBuffers &m_bufferAllocator;

      JetByteTools::WebSocket::HyBi::CProtocolHandlerAllocator m_protocolHandlerAllocator;
};

#endif // GATEWAY_SERVER_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
