#pragma once
#ifndef WEB_SOCKET_ECHO_SERVER_TEST_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
#define WEB_SOCKET_ECHO_SERVER_TEST_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2011 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/OpenSSLTools/StreamSocketConnectionManager.h"
#include "JetByteTools/OpenSSLTools/StreamSocketConnectionManagerCallback.h"

#include "JetByteTools/SocketTools/ReadTimeoutStreamSocketConnectionFilter.h"

#include "ServerCommon/ThreadedCallbackTimerQueue.h"

#include "JetByteTools/CoreTools/LockableObject.h"

#include "JetByteTools/WebSocketTools/Hixie76/IWebSocketServer.h"
#include "JetByteTools/WebSocketTools/Hixie76/ProtocolHandlerAllocator.h"
#include "JetByteTools/WebSocketTools/HyBi/IWebSocketServer.h"
#include "JetByteTools/WebSocketTools/HyBi/ProtocolHandlerAllocator.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// CStreamSocketConnectionManager
///////////////////////////////////////////////////////////////////////////////

class CStreamSocketConnectionManager :
   public JetByteTools::OpenSSL::CStreamSocketConnectionManager,
   private JetByteTools::Core::IQueueTimers::Timer,
   private JetByteTools::OpenSSL::CStreamSocketConnectionManagerCallback,
   private JetByteTools::Socket::CReadTimeoutStreamSocketConnectionFilter::TimerCallback,
   private JetByteTools::WebSocket::Hixie76::IWebSocketServer,
   private JetByteTools::WebSocket::HyBi::IWebSocketServer
{
   public :

      CStreamSocketConnectionManager(
         bool sslActive,
         const JetByteTools::OpenSSL::CContext &sslContext,
         bool verifyPeer,
         bool useHixie,
         JetByteTools::Milliseconds readTimeout,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator);

      CStreamSocketConnectionManager(
         const CStreamSocketConnectionManager &rhs) = delete;

      ~CStreamSocketConnectionManager();

      CStreamSocketConnectionManager &operator=(
         const CStreamSocketConnectionManager &rhs) = delete;

      bool DumpErrors() const;

   private :

      void OnPreOutgoingConnect(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const JetByteTools::Socket::IAddress &address,
         const void *pUserData) override;

      void OnSecureConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnOutgoingConnectionFailed(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         DWORD lastError) override;

      int Verify(
         int preverify_ok,
         X509_STORE_CTX *pContext) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionClientClose(
          JetByteTools::Socket::IStreamSocket &socket) override;

      void OnConnectionReset(
         JetByteTools::Socket::IStreamSocket &socket,
         DWORD lastError) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      void OnError(
         const JetByteTools::Core::_tstring &message,
         DWORD lastError) override;

      void OnConnectionClosure(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::Socket::ConnectionClosureReason reason) override;

      using JetByteTools::OpenSSL::CStreamSocketConnectionManagerCallback::OnError;

      // Implement Hixie76 IWebSocketServer

      void OnConnectionEstablished(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const std::string &uri,
         bool secure) override;

      void OnOutboundConnectionEstablished(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      void OnData(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         JetByteTools::WebSocket:: MessageStatus status) override;

      void OnData(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const JetByteTools::Core::_tstring &text,
         JetByteTools::WebSocket::MessageStatus status) override;

      void OnClientClose(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket) override;

      // Implement HyBi IWebSocketServer

      void OnOutboundConnectionEstablished(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      JetByteTools::WebSocket::ConnectionEstablishmentResult OnConnectionHandshake(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const std::string &uri,
         bool secure,
         const JetByteTools::WebSocket::CHeaders &requestHeaders,
         JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      void OnConnectionEstablished(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const std::string &uri,
         bool secure) override;

      void OnData(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::Core::_tstring &text,
         JetByteTools::WebSocket::MessageStatus status,
         __int64 messageBytesOutstanding) override;

      void OnData(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         JetByteTools::WebSocket::MessageType type,
         JetByteTools::WebSocket::MessageStatus status,
         __int64 messageBytesOutstanding) override;

      void OnPingResponse(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const BYTE *pData,
         BYTE length) override;

      void OnClientClose(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         WORD status,
         const JetByteTools::Core::_tstring &text) override;

      void OnError(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::Core::_tstring &message) override;

      void OnClosed(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         WORD status) override;

      // Implement IQueueTimers::Timer

      void OnTimer(
         JetByteTools::Core::IQueueTimers::Timer::UserData userData) override;

      // Implement CReadTimeoutStreamSocketConnectionFilter::TimerCallback

      void OnTimer(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::Socket::CReadTimeoutStreamSocketConnectionFilter::TimerCallback::UserData userData) override;

      void ProcessData(
         JetByteTools::WebSocket::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      void SendData(
         JetByteTools::WebSocket::IWebSocket &socket);

      void SetSendTimer(
         JetByteTools::WebSocket::IWebSocket &socket,
         JetByteTools::Milliseconds timeout);

      JetByteTools::IO::IAllocateBuffers &m_bufferAllocator;

      CThreadedCallbackTimerQueue m_timerQueue;

      JetByteTools::Core::CLockableObject m_lock;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_protocolHandlerIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_timerHandleIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_userDataIndex;

      typedef std::map<JetByteTools::Core::_tstring, size_t> StringErrors;

      typedef std::map<DWORD, size_t> LastErrors;

      LastErrors m_connectionErrors;
      LastErrors m_connectionResetErrors;

      StringErrors m_errors;

      bool DumpErrors(
         const JetByteTools::Core::_tstring &errorType,
         const LastErrors &lastErrors) const;

      JetByteTools::Socket::CReadTimeoutStreamSocketConnectionFilter m_timeoutFilter;

      const JetByteTools::Milliseconds m_readTimeout;

      JetByteTools::WebSocket::Hixie76::CProtocolHandlerAllocator m_hixieProtocolHandlerAllocator;
      JetByteTools::WebSocket::HyBi::CProtocolHandlerAllocator m_hyBiProtocolHandlerAllocator;

      const bool m_useHixie;
};

#endif // WEB_SOCKET_ECHO_SERVER_TEST_SOCKET_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
