///////////////////////////////////////////////////////////////////////////////
// File: CommandLine.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2011 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"
#include "JetByteTools/Admin/WinsockWrapper.h"

#include "CommandLine.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"

#pragma hdrstop

#include <iostream>

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;

using std::cout;
using std::endl;

///////////////////////////////////////////////////////////////////////////////
// Constants
///////////////////////////////////////////////////////////////////////////////

static const _tstring s_requireClientCertsSwitch   = _T("requireClientCerts");
static const _tstring s_certificatePathArgument    = _T("certificatePath");
static const _tstring s_serverKeyPasswordArgument  = _T("password");

///////////////////////////////////////////////////////////////////////////////
// CWebSocketServerCommandLine
///////////////////////////////////////////////////////////////////////////////

CWebSocketServerCommandLine::CWebSocketServerCommandLine(
   const DWORD argc,
   const char **argv)
   :  CCommandLine(
         argc,
         argv,
         _T("WebSocketServer"),
         CCommandLine::TCPServer | CCommandLine::DoesNotSupportWelcomeMessage)
{
   m_parser.AddSwitch(s_requireClientCertsSwitch);

   m_parser.AddArgument(s_certificatePathArgument);
   m_parser.AddArgument(s_serverKeyPasswordArgument);
}

bool CWebSocketServerCommandLine::RequireClientCerts() const
{
   return m_parser.IsSet(s_requireClientCertsSwitch);
}

_tstring CWebSocketServerCommandLine::CertificatePath() const
{
   return m_parser.GetOptionalValueAsString(s_certificatePathArgument, _T(".\\"));
}

_tstring CWebSocketServerCommandLine::Password() const
{
   return m_parser.GetOptionalValueAsString(s_serverKeyPasswordArgument, _T("password"));
}

void CWebSocketServerCommandLine::ShowMoreHelp()
{
   cout << " o -requireClientCerts" << endl;
   cout << " o -certificatePath    Defaults to .\\" << endl;
   cout << " o -password           The password used on the server's key." << endl;
   cout << "                       Defaults to \"password\"" << endl;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: CommandLine.cpp
///////////////////////////////////////////////////////////////////////////////
