#pragma once
#ifndef WEB_SOCKET_SERVER_HIXIE_SOCKET_SERVER_INCLUDED__
#define WEB_SOCKET_SERVER_HIXIE_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: HixieSocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2011 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "JetByteTools/CoreTools/IIndexedOpaqueUserData.h"

#include "JetByteTools/SocketTools/IAllocateSequencedStreamSockets.h"

#include "JetByteTools/WebSocketTools/Hixie76/ProtocolHandlerAllocator.h"

#include "JetByteTools/WebSocketTools/Hixie76/IWebSocketServer.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

///////////////////////////////////////////////////////////////////////////////
// CHixieSocketServer
///////////////////////////////////////////////////////////////////////////////

class CHixieSocketServer :
   public JetByteTools::Socket::CStreamSocketServerEx,
   private CStreamSocketServerExCallback,
   private JetByteTools::WebSocket::Hixie76::IWebSocketServer
{
   public :

      CHixieSocketServer(
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      ~CHixieSocketServer();

   private :

      /// Implement just the bits of IStreamSocketServerExCallback that we need...

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionReset(
          JetByteTools::Socket::IStreamSocket &socket,
          const DWORD lastError) override;

      void OnConnectionClosure(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::ConnectionClosureReason reason) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      /// Implement IAcceptWebSocketConnections

      JetByteTools::WebSocket::ConnectionEstablishmentResult OnConnectionHandshake(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const std::string &uri,
         const bool secure,
         const JetByteTools::WebSocket::CHeaders &requestHeaders,
         JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      /// Implement Hixie76 IWebSocketServer

      void OnConnectionEstablished(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const std::string &uri,
         const bool secure) override;

      void OnOutboundConnectionEstablished(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      void OnData(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         const JetByteTools::WebSocket::MessageStatus status) override;

      void OnData(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const JetByteTools::Core::_tstring &text,
         const JetByteTools::WebSocket::MessageStatus status) override;

      void OnClientClose(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket) override;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_protocolHandlerIndex;

      JetByteTools::IO::IAllocateBuffers &m_bufferAllocator;

      JetByteTools::WebSocket::Hixie76::CProtocolHandlerAllocator m_protocolHandlerAllocator;

      /// No copies do not implement
      CHixieSocketServer(const CHixieSocketServer &rhs);
      /// No copies do not implement
      CHixieSocketServer &operator=(const CHixieSocketServer &rhs);
};

#endif // WEB_SOCKET_SERVER_HIXIE_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: HixieSocketServer.h
///////////////////////////////////////////////////////////////////////////////
