#pragma once
#ifndef WEB_SOCKET_SERVER_SOCKET_SERVER_INCLUDED__
#define WEB_SOCKET_SERVER_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2011 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"

#include "JetByteTools/CoreTools/IIndexedOpaqueUserData.h"

#include "JetByteTools/SocketTools/IAllocateSequencedStreamSockets.h"

#include "JetByteTools/WebSocketTools/AutoDetectProtocolHandlerAllocator.h"

#include "JetByteTools/WebSocketTools/Hixie76/IWebSocketServer.h"
#include "JetByteTools/WebSocketTools/HyBi/IWebSocketServer.h"

#include "ServerCommon/StreamSocketServerExCallback.h"

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::Socket::CStreamSocketServerEx,
   private CStreamSocketServerExCallback,
   private JetByteTools::WebSocket::Hixie76::IWebSocketServer,
   private JetByteTools::WebSocket::HyBi::IWebSocketServer
{
   public :

      CSocketServer(
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::ILimitConnections &connectionLimiter = JetByteTools::Socket::CConnectionLimiter::NoLimitLimiter);

      ~CSocketServer();

   private :

      /// Implement just the bits of IStreamSocketServerExCallback that we need...

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionReset(
          JetByteTools::Socket::IStreamSocket &socket,
          const DWORD lastError) override;

      void OnConnectionClosure(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::ConnectionClosureReason reason) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      /// Implement IAcceptWebSocketConnections

      JetByteTools::WebSocket::ConnectionEstablishmentResult OnConnectionHandshake(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const std::string &uri,
         const bool secure,
         const JetByteTools::WebSocket::CHeaders &requestHeaders,
         JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      // Protocol agnostic functions

      void OnConnectionEstablished(
         JetByteTools::WebSocket::IWebSocket &socket,
         const std::string &uri,
         const bool secure);

      void OnMessage(
         JetByteTools::WebSocket::IWebSocket &socket,
         const JetByteTools::Core::_tstring &text);

      void OnMessage(
         JetByteTools::WebSocket::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      /// Implement Hixie76 IWebSocketServer

      void OnOutboundConnectionEstablished(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      void OnConnectionEstablished(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const std::string &uri,
         const bool secure) override;

      void OnData(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         const JetByteTools::WebSocket::MessageStatus status) override;

      void OnData(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket,
         const JetByteTools::Core::_tstring &text,
         const JetByteTools::WebSocket::MessageStatus status) override;

      void OnClientClose(
         JetByteTools::WebSocket::Hixie76::IWebSocket &socket) override;

      // Implement HyBi IWebSocketServer

      void OnOutboundConnectionEstablished(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::WebSocket::CHeaders &responseHeaders) override;

      void OnConnectionEstablished(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const std::string &uri,
         const bool secure) override;

      void OnData(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::Core::_tstring &text,
         const JetByteTools::WebSocket::MessageStatus status,
         const __int64 messageBytesOutstanding) override;

      void OnData(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         const JetByteTools::WebSocket::MessageType type,
         const JetByteTools::WebSocket::MessageStatus status,
         const __int64 messageBytesOutstanding) override;

      void OnPingResponse(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const BYTE *pData,
         const BYTE length) override;

      void OnClientClose(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const WORD status,
         const JetByteTools::Core::_tstring &text) override;

      void OnError(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const JetByteTools::Core::_tstring &message) override;

      void OnClosed(
         JetByteTools::WebSocket::HyBi::IWebSocket &socket,
         const WORD status) override;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_protocolHandlerIndex;

      JetByteTools::IO::IAllocateBuffers &m_bufferAllocator;

      JetByteTools::WebSocket::CAutoDetectProtocolHandlerAllocator m_protocolHandlerAllocator;

      /// No copies do not implement
      CSocketServer(const CSocketServer &rhs);
      /// No copies do not implement
      CSocketServer &operator=(const CSocketServer &rhs);
};

#endif // WEB_SOCKET_SERVER_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
