@Echo off

REM Most arguments passed to us via the environment as they contain spaces...

if not defined TEST_NAME goto NotSet
if not defined SERVER goto NotSet
if not defined SERVER_ARGS goto NotSet
if not defined OUT_DIR goto NotSet
if not defined TEST_CLIENT goto NotSet
if not defined TEST_CLIENT_ARGS goto NotSet

goto DisplayDebug

:NotSet
Echo One or more of the following variables is not set:
Echo -
Echo TEST_NAME=%TEST_NAME%
Echo SERVER=%SERVER%
Echo SERVER_ARGS=%SERVER_ARGS%
Echo TEST_CLIENT=%TEST_CLIENT%
Echo TEST_CLIENT_ARGS=%TEST_CLIENT_ARGS%
Echo -
Echo Please ensure they are all set in the environment before calling.
Exit /b 1

:DisplayDebug

if not defined JETBYTE_DEBUG_BUILD_SCRIPTS goto NoDebug

Echo Script debug information - RunTest.bat
Echo - 
Echo Required: TEST_NAME=%TEST_NAME%
Echo Required: SERVER=%SERVER%
Echo Required: OUT_DIR=%OUT_DIR%
Echo Required: SERVER_ARGS=%SERVER_ARGS%
Echo Required: TEST_CLIENT=%TEST_CLIENT%
Echo Required: TEST_CLIENT_ARGS=%TEST_CLIENT_ARGS%
Echo Required: SERVER_DIR=%SERVER_DIR%
Echo Optional: CHANGE_TO_WORKING_DIR=%CHANGE_TO_WORKING_DIR%
Echo Optional: WORKING_DIR_IS_SERVER_DIR=%WORKING_DIR_IS_SERVER_DIR%
Echo Optional: TEST_RUNNER=%TEST_RUNNER%
Echo Optional: TEST_RUNNER_ARGS=%TEST_RUNNER_ARGS%
Echo Optional: TEST_RUNNER_ID_GENERATOR=%TEST_RUNNER_ID_GENERATOR%
Echo Optional: TEST_RUNNER_WAITER=%TEST_RUNNER_WAITER%
Echo Optional: CLIENT_TEST_RUNNER=%CLIENT_TEST_RUNNER%
Echo Optional: CLIENT_TEST_RUNNER_ARGS=%CLIENT_TEST_RUNNER_ARGS%
Echo -

:NoDebug

setlocal

Echo Starting Test: %TEST_NAME%
Echo -

if not defined SERVER_DIR set SERVER_DIR=%OUT_DIR%

if defined TEST_RUNNER goto UseTestRunner

if defined CHANGE_TO_WORKING_DIR pushd %CHANGE_TO_WORKING_DIR%

if defined WORKING_DIR_IS_SERVER_DIR set SERVER_DIR=.\

Echo Starting %SERVER_DIR%\%SERVER% %SERVER_ARGS%

start %SERVER_DIR%\%SERVER% %SERVER_ARGS%

if defined CHANGE_TO_WORKING_DIR popd

goto WaitForServerStart

:UseTestRunner

if not defined TEST_RUNNER_ID_GENERATOR goto SkipTestRunGenerator

%TEST_RUNNER_ID_GENERATOR%

:SkipTestRunGenerator

start %TEST_RUNNER% -exe "%SERVER_DIR%\%SERVER%" -args "%SERVER_ARGS%" %TEST_RUNNER_ARGS%

:WaitForServerStart

Echo Waiting for server start up to complete
..\..\Test\ServerShutdown.exe WAIT_FOR_START
IF ERRORLEVEL 1 goto FailedToStart

Echo Server running
Echo -

if not defined CLIENT_TEST_RUNNER goto RunClientWithoutTestRunner

Echo Starting client - %CLIENT_TEST_RUNNER% -exe %TEST_CLIENT% -args "%TEST_CLIENT_ARGS%" %CLIENT_TEST_RUNNER_ARGS%
Echo -

%CLIENT_TEST_RUNNER% -exe %TEST_CLIENT% -args "%TEST_CLIENT_ARGS%" %CLIENT_TEST_RUNNER_ARGS%
IF ERRORLEVEL 1 goto ClientFailed

goto ClientCompleted

:RunClientWithoutTestRunner

Echo Starting client - %TEST_CLIENT% %TEST_CLIENT_ARGS%
Echo -

%TEST_CLIENT% %TEST_CLIENT_ARGS%
IF ERRORLEVEL 1 goto ClientFailed

:ClientCompleted

Echo Client completed
Echo -

Echo Shutting down server
Echo -

..\..\Test\ServerShutdown.exe WAIT_FOR_STOP
REM failed to stop??

if not defined TEST_RUNNER_WAITER goto TestPassed

Echo Waiting for test runner to complete
Echo -

%TEST_RUNNER_WAITER%
IF ERRORLEVEL 1 goto TestFailed

:TestPassed

Echo Test Passed: %TEST_NAME%
Echo -

goto End

:ClientFailed

Echo Test Failed: %TEST_NAME%
Echo Client failed.
Echo ERRORLEVEL=%ERRORLEVEL%
Echo -

if not defined TEST_RUNNER_WAITER goto ForceShutdown

Echo Shutting down server
Echo -

..\..\Test\ServerShutdown.exe WAIT_FOR_STOP
REM failed to stop??

Echo Waiting for test runner to complete
Echo -

%TEST_RUNNER_WAITER%

goto ForceShutdown

:TestFailed

Echo Test Failed: %TEST_NAME%
Echo Server failed.
Echo ERRORLEVEL=%ERRORLEVEL%
Echo -

:ForceShutdown

Echo Attempting to force the server to shut down
Echo -

..\..\Test\ServerShutdown.exe WAIT_FOR_STOP

Exit /b 1

:FailedToStart

Echo Server Failed to start: %TEST_NAME%

if not defined TEST_RUNNER_WAITER Exit /b 1

Echo Waiting for test runner to complete
Echo -

%TEST_RUNNER_WAITER%

Exit /b 1

:End
